;; Copyright (C) 2019  Brandon M. Invergo <brandon@invergo.net>

;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

(define-module (librekontrol input)
  #:export (make-input-event
            define-input-event))

(define-syntax-rule (make-input-event event-type event-code)
    "Construct an input event defined by the EVENT-TYPE and
EVENT-CODE.  For now, at least, this is represented as a simple pair 
(EVENT-TYPE . EVENT-CODE)."
  (cons event-type event-code))

(define-syntax-rule (define-input-event symbol event-type event-code)
  "Bind SYMBOL to a specific input event defined by the EVENT-TYPE and
EVENT-CODE"
  (define symbol (make-input-event event-type event-code)))

;; Input event types
(define-public ev-syn #x00)
(define-public ev-key #x01)
(define-public ev-rel #x02)
(define-public ev-abs #x03)
(define-public ev-msc #x04)
(define-public ev-sw #x05)

;; Input event codes
(define-public key-esc 1)
(define-public key-1 2)
(define-public key-2 3)
(define-public key-3 4)
(define-public key-4 5)
(define-public key-5 6)
(define-public key-6 7)
(define-public key-7 8)
(define-public key-8 9)
(define-public key-9 10)
(define-public key-0 11)
(define-public key-minus 12)
(define-public key-equal 13)
(define-public key-backspace 14)
(define-public key-tab 15)
(define-public key-q 16)
(define-public key-w 17)
(define-public key-e 18)
(define-public key-r 19)
(define-public key-t 20)
(define-public key-y 21)
(define-public key-u 22)
(define-public key-i 23)
(define-public key-o 24)
(define-public key-p 25)
(define-public key-leftbrace 26)
(define-public key-rightbrace 27)
(define-public key-enter 28)
(define-public key-leftctrl 29)
(define-public key-a 30)
(define-public key-s 31)
(define-public key-d 32)
(define-public key-f 33)
(define-public key-g 34)
(define-public key-h 35)
(define-public key-j 36)
(define-public key-k 37)
(define-public key-l 38)
(define-public key-semicolon 39)
(define-public key-apostrophe 40)
(define-public key-grave 41)
(define-public key-leftshift 42)
(define-public key-backslash 43)
(define-public key-z 44)
(define-public key-x 45)
(define-public key-c 46)
(define-public key-v 47)
(define-public key-b 48)
(define-public key-n 49)
(define-public key-m 50)
(define-public key-comma 51)
(define-public key-dot 52)
(define-public key-slash 53)
(define-public key-rightshift 54)
(define-public key-kpasterisk 55)
(define-public key-leftalt 56)
(define-public key-space 57)
(define-public key-capslock 58)
(define-public key-f1 59)
(define-public key-f2 60)
(define-public key-f3 61)
(define-public key-f4 62)
(define-public key-f5 63)
(define-public key-f6 64)
(define-public key-f7 65)
(define-public key-f8 66)
(define-public key-f9 67)
(define-public key-f10 68)
(define-public key-numlock 69)
(define-public key-scrolllock 70)
(define-public key-kp7 71)
(define-public key-kp8 72)
(define-public key-kp9 73)
(define-public key-kpminus 74)
(define-public key-kp4 75)
(define-public key-kp5 76)
(define-public key-kp6 77)
(define-public key-kpplus 78)
(define-public key-kp1 79)
(define-public key-kp2 80)
(define-public key-kp3 81)
(define-public key-kp0 82)
(define-public key-kpdot 83)
(define-public key-zenkakuhankaku 85)
(define-public key-102nd 86)
(define-public key-f11 87)
(define-public key-f12 88)
(define-public key-ro 89)
(define-public key-katakana 90)
(define-public key-hiragana 91)
(define-public key-henkan 92)
(define-public key-katakanahiragana 93)
(define-public key-muhenkan 94)
(define-public key-kpjpcomma 95)
(define-public key-kpenter 96)
(define-public key-rightctrl 97)
(define-public key-kpslash 98)
(define-public key-sysrq 99)
(define-public key-rightalt 100)
(define-public key-linefeed 101)
(define-public key-home 102)
(define-public key-up 103)
(define-public key-pageup 104)
(define-public key-left 105)
(define-public key-right 106)
(define-public key-end 107)
(define-public key-down 108)
(define-public key-pagedown 109)
(define-public key-insert 110)
(define-public key-delete 111)
(define-public key-macro 112)
(define-public key-mute 113)
(define-public key-volumedown 114)
(define-public key-volumeup 115)
(define-public key-power 116)
(define-public key-kpequal 117)
(define-public key-kpplusminus 118)
(define-public key-pause 119)
(define-public key-scale 120)
(define-public key-kpcomma 121)
(define-public key-hangeul 122)
(define-public key-hanguel 122)
(define-public key-hanja 123)
(define-public key-yen 124)
(define-public key-leftmeta 125)
(define-public key-rightmeta 126)
(define-public key-compose 127)
(define-public key-stop 128)
(define-public key-again 129)
(define-public key-props 130)
(define-public key-undo 131)
(define-public key-front 132)
(define-public key-copy 133)
(define-public key-open 134)
(define-public key-paste 135)
(define-public key-find 136)
(define-public key-cut 137)
(define-public key-help 138)
(define-public key-menu 139)
(define-public key-calc 140)
(define-public key-setup 141)
(define-public key-sleep 142)
(define-public key-wakeup 143)
(define-public key-file 144)
(define-public key-sendfile 145)
(define-public key-deletefile 146)
(define-public key-xfer 147)
(define-public key-prog1 148)
(define-public key-prog2 149)
(define-public key-www 150)
(define-public key-msdos 151)
(define-public key-coffee 152)
(define-public key-screenlock 152)
(define-public key-rotate-display 153)
(define-public key-direction 153)
(define-public key-cyclewindows 154)
(define-public key-mail 155)
(define-public key-bookmarks 156)
(define-public key-computer 157)
(define-public key-back 158)
(define-public key-forward 159)
(define-public key-closecd 160)
(define-public key-ejectcd 161)
(define-public key-ejectclosecd 162)
(define-public key-nextsong 163)
(define-public key-playpause 164)
(define-public key-previoussong 165)
(define-public key-stopcd 166)
(define-public key-record 167)
(define-public key-rewind 168)
(define-public key-phone 169)
(define-public key-iso 170)
(define-public key-config 171)
(define-public key-homepage 172)
(define-public key-refresh 173)
(define-public key-exit 174)
(define-public key-move 175)
(define-public key-edit 176)
(define-public key-scrollup 177)
(define-public key-scrolldown 178)
(define-public key-kpleftparen 179)
(define-public key-kprightparen 180)
(define-public key-new 181)
(define-public key-redo 182)
(define-public key-f13 183)
(define-public key-f14 184)
(define-public key-f15 185)
(define-public key-f16 186)
(define-public key-f17 187)
(define-public key-f18 188)
(define-public key-f19 189)
(define-public key-f20 190)
(define-public key-f21 191)
(define-public key-f22 192)
(define-public key-f23 193)
(define-public key-f24 194)
(define-public key-playcd 200)
(define-public key-pausecd 201)
(define-public key-prog3 202)
(define-public key-prog4 203)
(define-public key-dashboard 204)
(define-public key-suspend 205)
(define-public key-close 206)
(define-public key-play 207)
(define-public key-fastforward 208)
(define-public key-bassboost 209)
(define-public key-print 210)
(define-public key-hp 211)
(define-public key-camera 212)
(define-public key-sound 213)
(define-public key-question 214)
(define-public key-email 215)
(define-public key-chat 216)
(define-public key-search 217)
(define-public key-connect 218)
(define-public key-finance 219)
(define-public key-sport 220)
(define-public key-shop 221)
(define-public key-alterase 222)
(define-public key-cancel 223)
(define-public key-brightnessdown 224)
(define-public key-brightnessup 225)
(define-public key-media 226)
(define-public key-switchvideomode 227)
(define-public key-kbdillumtoggle 228)
(define-public key-kbdillumdown 229)
(define-public key-kbdillumup 230)
(define-public key-send 231)
(define-public key-reply 232)
(define-public key-forwardmail 233)
(define-public key-save 234)
(define-public key-documents 235)
(define-public key-battery 236)
(define-public key-bluetooth 237)
(define-public key-wlan 238)
(define-public key-uwb 239)
(define-public key-unknown 240)
(define-public key-video-next 241)
(define-public key-video-prev 242)
(define-public key-brightness-cycle 243)
(define-public key-brightness-auto 244)
(define-public key-brightness-zero 244)
(define-public key-display-off 245)
(define-public key-wwan 246)
(define-public key-wimax 246)
(define-public key-rfkill 247)
(define-public key-micmute 248)
(define-public btn-misc #x100)
(define-public btn-0 #x100)
(define-public btn-1 #x101)
(define-public btn-2 #x102)
(define-public btn-3 #x103)
(define-public btn-4 #x104)
(define-public btn-5 #x105)
(define-public btn-6 #x106)
(define-public btn-7 #x107)
(define-public btn-8 #x108)
(define-public btn-9 #x109)
(define-public btn-mouse #x110)
(define-public btn-left #x110)
(define-public btn-right #x111)
(define-public btn-middle #x112)
(define-public btn-side #x113)
(define-public btn-extra #x114)
(define-public btn-forward #x115)
(define-public btn-back #x116)
(define-public btn-task #x117)
(define-public btn-joystick #x120)
(define-public btn-trigger #x120)
(define-public btn-thumb #x121)
(define-public btn-thumb2 #x122)
(define-public btn-top #x123)
(define-public btn-top2 #x124)
(define-public btn-pinkie #x125)
(define-public btn-base #x126)
(define-public btn-base2 #x127)
(define-public btn-base3 #x128)
(define-public btn-base4 #x129)
(define-public btn-base5 #x12a)
(define-public btn-base6 #x12b)
(define-public btn-dead #x12f)
(define-public btn-gamepad #x130)
(define-public btn-south #x130)
(define-public btn-a #x130)
(define-public btn-east #x131)
(define-public btn-b #x131)
(define-public btn-c #x132)
(define-public btn-north #x133)
(define-public btn-x #x133)
(define-public btn-west #x134)
(define-public btn-y #x134)
(define-public btn-z #x135)
(define-public btn-tl #x136)
(define-public btn-tr #x137)
(define-public btn-tl2 #x138)
(define-public btn-tr2 #x139)
(define-public btn-select #x13a)
(define-public btn-start #x13b)
(define-public btn-mode #x13c)
(define-public btn-thumbl #x13d)
(define-public btn-thumbr #x13e)
(define-public btn-digi #x140)
(define-public btn-tool-pen #x140)
(define-public btn-tool-rubber #x141)
(define-public btn-tool-brush #x142)
(define-public btn-tool-pencil #x143)
(define-public btn-tool-airbrush #x144)
(define-public btn-tool-finger #x145)
(define-public btn-tool-mouse #x146)
(define-public btn-tool-lens #x147)
(define-public btn-tool-quinttap #x148)
(define-public btn-stylus3 #x149)
(define-public btn-touch #x14a)
(define-public btn-stylus #x14b)
(define-public btn-stylus2 #x14c)
(define-public btn-tool-doubletap #x14d)
(define-public btn-tool-tripletap #x14e)
(define-public btn-tool-quadtap #x14f)
(define-public btn-wheel #x150)
(define-public btn-gear-down #x150)
(define-public btn-gear-up #x151)

(define-public key-ok #x160)
(define-public key-select #x161)
(define-public key-goto #x162)
(define-public key-clear #x163)
(define-public key-power2 #x164)
(define-public key-option #x165)
(define-public key-info #x166)
(define-public key-time #x167)
(define-public key-vendor #x168)
(define-public key-archive #x169)
(define-public key-program #x16a)
(define-public key-channel #x16b)
(define-public key-favorites #x16c)
(define-public key-epg #x16d)
(define-public key-pvr #x16e)
(define-public key-mhp #x16f)
(define-public key-language #x170)
(define-public key-title #x171)
(define-public key-subtitle #x172)
(define-public key-angle #x173)
(define-public key-zoom #x174)
(define-public key-mode #x175)
(define-public key-keyboard #x176)
(define-public key-screen #x177)
(define-public key-pc #x178)
(define-public key-tv #x179)
(define-public key-tv2 #x17a)
(define-public key-vcr #x17b)
(define-public key-vcr2 #x17c)
(define-public key-sat #x17d)
(define-public key-sat2 #x17e)
(define-public key-cd #x17f)
(define-public key-tape #x180)
(define-public key-radio #x181)
(define-public key-tuner #x182)
(define-public key-player #x183)
(define-public key-text #x184)
(define-public key-dvd #x185)
(define-public key-aux #x186)
(define-public key-mp3 #x187)
(define-public key-audio #x188)
(define-public key-video #x189)
(define-public key-directory #x18a)
(define-public key-list #x18b)
(define-public key-memo #x18c)
(define-public key-calendar #x18d)
(define-public key-red #x18e)
(define-public key-green #x18f)
(define-public key-yellow #x190)
(define-public key-blue #x191)
(define-public key-channelup #x192)
(define-public key-channeldown #x193)
(define-public key-first #x194)
(define-public key-last #x195)
(define-public key-ab #x196)
(define-public key-next #x197)
(define-public key-restart #x198)
(define-public key-slow #x199)
(define-public key-shuffle #x19a)
(define-public key-break #x19b)
(define-public key-previous #x19c)
(define-public key-digits #x19d)
(define-public key-teen #x19e)
(define-public key-twen #x19f)
(define-public key-videophone #x1a0)
(define-public key-games #x1a1)
(define-public key-zoomin #x1a2)
(define-public key-zoomout #x1a3)
(define-public key-zoomreset #x1a4)
(define-public key-wordprocessor #x1a5)
(define-public key-editor #x1a6)
(define-public key-spreadsheet #x1a7)
(define-public key-graphicseditor #x1a8)
(define-public key-presentation #x1a9)
(define-public key-database #x1aa)
(define-public key-news #x1ab)
(define-public key-voicemail #x1ac)
(define-public key-addressbook #x1ad)
(define-public key-messenger #x1ae)
(define-public key-displaytoggle #x1af)
(define-public key-brightness-toggle #x1af)
(define-public key-spellcheck #x1b0)
(define-public key-logoff #x1b1)
(define-public key-dollar #x1b2)
(define-public key-euro #x1b3)
(define-public key-frameback #x1b4)
(define-public key-frameforward #x1b5)
(define-public key-context-menu #x1b6)
(define-public key-media-repeat #x1b7)
(define-public key-10channelsup #x1b8)
(define-public key-10channelsdown #x1b9)
(define-public key-images #x1ba)
(define-public key-del-eol #x1c0)
(define-public key-del-eos #x1c1)
(define-public key-ins-line #x1c2)
(define-public key-del-line #x1c3)
(define-public key-fn #x1d0)
(define-public key-fn-esc #x1d1)
(define-public key-fn-f1 #x1d2)
(define-public key-fn-f2 #x1d3)
(define-public key-fn-f3 #x1d4)
(define-public key-fn-f4 #x1d5)
(define-public key-fn-f5 #x1d6)
(define-public key-fn-f6 #x1d7)
(define-public key-fn-f7 #x1d8)
(define-public key-fn-f8 #x1d9)
(define-public key-fn-f9 #x1da)
(define-public key-fn-f10 #x1db)
(define-public key-fn-f11 #x1dc)
(define-public key-fn-f12 #x1dd)
(define-public key-fn-1 #x1de)
(define-public key-fn-2 #x1df)
(define-public key-fn-d #x1e0)
(define-public key-fn-e #x1e1)
(define-public key-fn-f #x1e2)
(define-public key-fn-s #x1e3)
(define-public key-fn-b #x1e4)
(define-public key-brl-dot1 #x1f1)
(define-public key-brl-dot2 #x1f2)
(define-public key-brl-dot3 #x1f3)
(define-public key-brl-dot4 #x1f4)
(define-public key-brl-dot5 #x1f5)
(define-public key-brl-dot6 #x1f6)
(define-public key-brl-dot7 #x1f7)
(define-public key-brl-dot8 #x1f8)
(define-public key-brl-dot9 #x1f9)
(define-public key-brl-dot10 #x1fa)
(define-public key-numeric-0 #x200)
(define-public key-numeric-1 #x201)
(define-public key-numeric-2 #x202)
(define-public key-numeric-3 #x203)
(define-public key-numeric-4 #x204)
(define-public key-numeric-5 #x205)
(define-public key-numeric-6 #x206)
(define-public key-numeric-7 #x207)
(define-public key-numeric-8 #x208)
(define-public key-numeric-9 #x209)
(define-public key-numeric-star #x20a)
(define-public key-numeric-pound #x20b)
(define-public key-numeric-a #x20c)
(define-public key-numeric-b #x20d)
(define-public key-numeric-c #x20e)
(define-public key-numeric-d #x20f)
(define-public key-camera-focus #x210)
(define-public key-wps-button #x211)
(define-public key-touchpad-toggle #x212)
(define-public key-touchpad-on #x213)
(define-public key-touchpad-off #x214)
(define-public key-camera-zoomin #x215)
(define-public key-camera-zoomout #x216)
(define-public key-camera-up #x217)
(define-public key-camera-down #x218)
(define-public key-camera-left #x219)
(define-public key-camera-right #x21a)
(define-public key-attendant-on #x21b)
(define-public key-attendant-off #x21c)
(define-public key-attendant-toggle #x21d)
(define-public key-lights-toggle #x21e)
(define-public btn-dpad-up #x220)
(define-public btn-dpad-down #x221)
(define-public btn-dpad-left #x222)
(define-public btn-dpad-right #x223)
(define-public key-als-toggle #x230)
(define-public key-rotate-lock-toggle #x231)
(define-public key-buttonconfig #x240)
(define-public key-taskmanager #x241)
(define-public key-journal #x242)
(define-public key-controlpanel #x243)
(define-public key-appselect #x244)
(define-public key-screensaver #x245)
(define-public key-voicecommand #x246)
(define-public key-assistant #x247)
(define-public key-brightness-min #x250)
(define-public key-brightness-max #x251)
(define-public key-kbdinputassist-prev #x260)
(define-public key-kbdinputassist-next #x261)
(define-public key-kbdinputassist-prevgroup #x262)
(define-public key-kbdinputassist-nextgroup #x263)
(define-public key-kbdinputassist-accept #x264)
(define-public key-kbdinputassist-cancel #x265)
(define-public key-right-up #x266)
(define-public key-right-down #x267)
(define-public key-left-up #x268)
(define-public key-left-down #x269)
(define-public key-root-menu #x26a)
(define-public key-media-top-menu #x26b)
(define-public key-numeric-11 #x26c)
(define-public key-numeric-12 #x26d)
(define-public key-audio-desc #x26e)
(define-public key-3d-mode #x26f)
(define-public key-next-favorite #x270)
(define-public key-stop-record #x271)
(define-public key-pause-record #x272)
(define-public key-vod #x273)
(define-public key-unmute #x274)
(define-public key-fastreverse #x275)
(define-public key-slowreverse #x276)
(define-public key-data #x277)
(define-public key-onscreen-keyboard #x278)
(define-public btn-trigger-happy #x2c0)
(define-public btn-trigger-happy1 #x2c0)
(define-public btn-trigger-happy2 #x2c1)
(define-public btn-trigger-happy3 #x2c2)
(define-public btn-trigger-happy4 #x2c3)
(define-public btn-trigger-happy5 #x2c4)
(define-public btn-trigger-happy6 #x2c5)
(define-public btn-trigger-happy7 #x2c6)
(define-public btn-trigger-happy8 #x2c7)
(define-public btn-trigger-happy9 #x2c8)
(define-public btn-trigger-happy10 #x2c9)
(define-public btn-trigger-happy11 #x2ca)
(define-public btn-trigger-happy12 #x2cb)
(define-public btn-trigger-happy13 #x2cc)
(define-public btn-trigger-happy14 #x2cd)
(define-public btn-trigger-happy15 #x2ce)
(define-public btn-trigger-happy16 #x2cf)
(define-public btn-trigger-happy17 #x2d0)
(define-public btn-trigger-happy18 #x2d1)
(define-public btn-trigger-happy19 #x2d2)
(define-public btn-trigger-happy20 #x2d3)
(define-public btn-trigger-happy21 #x2d4)
(define-public btn-trigger-happy22 #x2d5)
(define-public btn-trigger-happy23 #x2d6)
(define-public btn-trigger-happy24 #x2d7)
(define-public btn-trigger-happy25 #x2d8)
(define-public btn-trigger-happy26 #x2d9)
(define-public btn-trigger-happy27 #x2da)
(define-public btn-trigger-happy28 #x2db)
(define-public btn-trigger-happy29 #x2dc)
(define-public btn-trigger-happy30 #x2dd)
(define-public btn-trigger-happy31 #x2de)
(define-public btn-trigger-happy32 #x2df)
(define-public btn-trigger-happy33 #x2e0)
(define-public btn-trigger-happy34 #x2e1)
(define-public btn-trigger-happy35 #x2e2)
(define-public btn-trigger-happy36 #x2e3)
(define-public btn-trigger-happy37 #x2e4)
(define-public btn-trigger-happy38 #x2e5)
(define-public btn-trigger-happy39 #x2e6)
(define-public btn-trigger-happy40 #x2e7)

(define-public key-max #x2ff)

(define-public rel-x #x00)
(define-public rel-y #x01)
(define-public rel-z #x02)
(define-public rel-rx #x03)
(define-public rel-ry #x04)
(define-public rel-rz #x05)
(define-public rel-hwheel #x06)
(define-public rel-dial #x07)
(define-public rel-wheel #x08)
(define-public rel-misc #x09)
(define-public rel-max #x0f)

(define-public abs-x #x00)
(define-public abs-y #x01)
(define-public abs-z #x02)
(define-public abs-rx #x03)
(define-public abs-ry #x04)
(define-public abs-rz #x05)
(define-public abs-throttle #x06)
(define-public abs-rudder #x07)
(define-public abs-wheel #x08)
(define-public abs-gas #x09)
(define-public abs-brake #x0a)
(define-public abs-hat0x #x10)
(define-public abs-hat0y #x11)
(define-public abs-hat1x #x12)
(define-public abs-hat1y #x13)
(define-public abs-hat2x #x14)
(define-public abs-hat2y #x15)
(define-public abs-hat3x #x16)
(define-public abs-hat3y #x17)
(define-public abs-pressure #x18)
(define-public abs-distance #x19)
(define-public abs-tilt-x #x1a)
(define-public abs-tilt-y #x1b)
(define-public abs-tool-width #x1c)
(define-public abs-volume #x20)
(define-public abs-misc #x28)
(define-public abs-mt-slot #x2f)
(define-public abs-mt-touch-major #x30)
(define-public abs-mt-touch-minor #x31)
(define-public abs-mt-width-major #x32)
(define-public abs-mt-width-minor #x33)
(define-public abs-mt-orientation #x34)
(define-public abs-mt-position-x #x35)
(define-public abs-mt-position-y #x36)
(define-public abs-mt-tool-type #x37)
(define-public abs-mt-blob-id #x38)
(define-public abs-mt-tracking-id #x39)
(define-public abs-mt-pressure #x3a)
(define-public abs-mt-distance #x3b)
(define-public abs-mt-tool-x #x3c)
(define-public abs-mt-tool-y #x3d)
(define-public abs-max #x3f)

(define-public sw-lid #x00)
(define-public sw-tablet-mode #x01)
(define-public sw-headphone-insert #x02)
(define-public sw-rfkill-all #x03)
(define-public sw-radio #x03)
(define-public sw-microphone-insert #x04)
(define-public sw-dock #x05)
(define-public sw-lineout-insert #x06)
(define-public sw-jack-physical-insert #x07)
(define-public sw-videoout-insert #x08)
(define-public sw-camera-lens-cover #x09)
(define-public sw-keypad-slide #x0a)
(define-public sw-front-proximity #x0b)
(define-public sw-rotate-lock #x0c)
(define-public sw-linein-insert #x0d)
(define-public sw-mute-device #x0e)
(define-public sw-pen-inserted #x0f)
(define-public sw-max #x0f)
